#!/usr/bin/env python

################################################################################
#
# Module:    LogFileTools.py
#
# Purpose:   Provide methods for processing trailer files.  
#               - Comparing steps in multiple trailers
#               - Processing time duration between steps
#
# Methods:   __init__            /Constructor
#            load_trail_mlist    /Loads in master list of keywords to search
#            load_trail_slist    /Loads in step list created by user
#            load_trail_files    /Loads in trailer files to extract from
#            get_timediff        /Calculates all time differences between start
#                                 tags and end tags
#            translate_time      /Converts time stamp into seconds to compare
#            translate_date      /Converts time stamp into standard date
#            print_chart         /Prints chart to output file
#            list_steps          /Lists steps found in log file
#            cmp_steps           /Compares existing steps in 2 log files
#
# Usage:     trailer_handle = LogFileTools.LogFileTools()
#
# History:
# Date      Who      Description
# ----      ----     -----------------------------------------------------------
# 11/17/10  RBelt    Initial Revision
# 02/23/10  RBelt    Added list_steps and cmp_steps functions
#
################################################################################

# Imports

import os,sys,re,time
from PipelineUtils import evaluateDefinitionsFile

class LogFileTools():

  def __init__(self):
    """ Constructor

        Set class variables

    """

    self.module = 'LogFileTools.py'
    self.mfile  = None
    self.sfile  = None
    self.slist  = {}
    self.mlist = {}

  ##############################################################################

  def load_trail_mlist(self,tra_files,mlist_file):
    """

    Description: This method will read in the master list files and the trailer
                 files.  This master list files should be in the format...
  
                 TAG       KEYWORD  -->  Number of spaces between is irrelevant
  
                 With each file it will fill the tag_mlist dictionary
                 tag_mlist[FILENAME][TAG VALUE]['keyword'] = KEYWORD VALUE
                                               ['time'] --> This will originally
                                                        set to 'None' but should
                                                        be initialized here
    Parameters:

                tra_files: List of trail files
                           This is used to fill in keys for the dictionary
                slist_file : Master list file

    Return Values:

                tag_mlist: Filled with trail file and master list file

    """
    self.mfile = mlist_file
    file_handle = open(mlist_file,'r')
    lines = file_handle.readlines()
    for file in tra_files:
      self.mlist[file] = {}
      for line in lines:
        if line[0] != '!':
          tag = line[0:line.find(' ')]
          line = line[len(tag):]
          srch_string = re.sub('^\s*','',line)
          srch_string = srch_string[:srch_string.find('\n')]
          self.mlist[file][tag] = {}
          self.mlist[file][tag]['keyword'] = srch_string
          self.mlist[file][tag]['time'] = ''
    return self.mlist

  ##############################################################################

  def load_trail_slist(self,tra_files,slist_file):
    """

    Description: This method will read in the master list files and the trailer files
                 to fill in slist_file.  The step list file should be in this format..

                 TAG       FIRST TAG - SECOND TAG

                 TAG --> Label you want the match to.  Spaces between TAG and FIRST TAG
                                                     can be variable.

                 FIRST TAG and SECOND TAG must be in format 'FIRST - SECOND'

    Parameters:

                tra_files: List of trail files
                           This is used fill in keys for the dictionary
                slist_file : Step list file

    Return Values:

                tag_slist: Filled with trail file and step list file

    """
    self.sfile = slist_file
    file_handle = open(slist_file,'r')
    lines = file_handle.readlines()
    slist_order_array = []
    for line in lines:
      if line[0] != '!':
        tag = line[0:line.find(' ')]
        slist_order_array.append(tag)
        line = line[len(tag):]
        srch_string = re.sub('^\s*','',line)
        combo = srch_string[:srch_string.find('\n')]
        start_tag = combo[:combo.find(' - ')]
        end_tag = combo[(combo.find(' - ')+3):]
        for file in tra_files:
          if(self.slist.has_key(file) is False):
            self.slist[file] = {}
          self.slist[file][tag] = {}
          self.slist[file][tag]['start_tag'] = start_tag
          self.slist[file][tag]['end_tag'] = end_tag
          self.slist[file][tag]['time_diff'] = " "
    self.slist['order'] = slist_order_array
    return self.slist

  ##############################################################################

  def load_trail_files(self,tra_files):
    """

    Description: Load in trail files line by line and extract times steps for each
                 label in the master list file.

                 *All tags in master list will be found.  Could possibly more time
                  efficient if code only found tags found in slist file as well.

                 Every tag found in mlist that has a time value in the trail file
                 will now have the time value assciated with it.  This allows any
                 sequence of 2 steps to be given a time difference.

    Parameters:  tra_files : List of trailer files

    Return Values : mlist : same dictionary but with time values loaded.

    """
    for file in tra_files:
      fh = open(file,'r')
      lines =fh.readlines()
      file_name = file[file.rfind('/')+1:]
      for line in lines:
        for key in self.mlist[file_name]:
          if(re.search(self.mlist[file_name][key]['keyword'],line)):
            try:
              int(line[:13])
              self.mlist[file_name][key]['time'] = line[:13]
            except:
              1
    return self.mlist
  ##############################################################################

  def get_timediff(self):
    """

    Description: To calculate the time differences between all start tags and end
                 tags within the slist dictionary.  The time differences will be
                 calculated from time values in mlist.  The slist time difference
                 value will be loaded into the slist dictionary.

    Parameters:  slist : Used to load in time_diff values
                 mlist : Used to get values for start tags and end tags to
                             get time difference

    Return Values: _mlist : Should have all time_diff values populated

    """
    for file in self.slist:
      if (file != 'order'):
        for tag in self.slist[file]:
          end = self.mlist[file][self.slist[file][tag]['end_tag']]['time']
          beg = self.mlist[file][self.slist[file][tag]['start_tag']]['time']
          if (end != '' and beg != ''):
            self.slist[file][tag]['time_diff'] = self.translate_time(end,'seconds') - self.translate_time(beg,'seconds')
    return self.mlist

  ################################################################################

  def translate_time(self,timeval,format):
    """

   Description: This method translates a 13 character time string into a
                 computable integer to get the time difference.  Currently, the
                 only format needed is time in seconds.  But shouldn't be too
                 difficult to update method to handle hours/minutes/days/etc.

    Parameters:  time   : Time value in 13 character string
                 format : Format of time you want to be returned
                          seconds/minutes/hours/days

    Return Values : integer value of time in format requested in parameters.

    """
    if (format == 'seconds'):
      year  = timeval[:4]
      days  = timeval[4:7]
      hours = timeval[7:9]
      mins  = timeval[9:11]
      secs  = timeval[11:13]
      time_str = year + " " + days + " " + hours + " " + mins + " " + secs
      time_in_secs = 0
      try:
        time_result = time.strptime(time_str, "%Y %j %H %M %S")
        time_in_secs = time.mktime(time_result)
      except ValueError:
        opusutil.PrintMsg("E",'Unable to evaluate time value',"trailer.py")
      return time_in_secs

  ################################################################################

  def translate_date(self,timeval):
    """

    Description: This method translates a 13 character time string into a
                 calendar date

    Parameters:  timeval: Time value in 13 character string

    Return Values : integer value of time in format requested in parameters.

    """
    if(timeval == ""):
      return timeval
    year  = timeval[:4]
    days  = timeval[4:7]
    hours = timeval[7:9]
    mins  = timeval[9:11]
    secs  = timeval[11:13]
    time_str = year + " " + days + " " + hours + " " + mins + " " + secs

    try:
      final_time = time.strftime("%b %d %Y %H:%M:%S", time.strptime(time_str, "%Y %j %H %M %S"))
      return final_time
    except ValueError:
      print "Could not evaluate time from string"
    return timeval

  ################################################################################

  def print_chart(self,tra_files,out_type,out_file):
    """

    Description: Prints output tags, files, and time values.  This can currently
                 be in 2 formats (excel, screen).  Currently screen print out is
                 a simple comma separated list.  Excel printout creates a CSV file
                 compatible with excel spreadsheet.

    Parameters:  tagslist  : to be printed
                 tra_files : will be printed and used as lookup keys in slist.

    Return Values: None / Result is printed

    """
    line = "File Name,"
    for key in self.slist['order']:
      line += ",," + key + ","
    line += "\n"
    line += ","
    for key in self.slist['order']:
      line += ",Start Time,End Time,Elapsed Time(secs)"
    line += "\n"
    for file in tra_files:
      line += file + ","
      for key in self.slist['order']:
        line += "," + self.translate_date(str(self.mlist[file][self.slist[file][key]['start_tag']]['time']))
        line += "," + self.translate_date(str(self.mlist[file][self.slist[file][key]['end_tag']]['time']))
        line += "," + str(self.slist[file][key]['time_diff'])
      line += "\n"
    if (out_type == "excel"):
      outfile = open(out_file,'w')
      outfile.write(line)
      outfile.close()
    elif (out_type == "screen"):
      print line

  ################################################################################

  def list_steps(self,file=None,mfile=None):
    """ Will return list of steps for a specific file.
      
    Parameters:    file : This is the file that will be processed.  If file is none,
                        it is presumed that self.mlist is already loaded and will
                        return information for all files loaded.  If file is not
                        none, the file will be loaded and the list will be returned
                        only for that file.

    Return Values: steps : Array that will be returned including all the steps that 
                           are included with a file.  If there is no file passed, 
                           then a dictionary of files/array of steps will be 
                           returned.

    """
    if(mfile == None):
      try:
        mfile = evaluateDefinitionsFile("trailer_master.lis")
      except:
        print "Unable to determine trailer_master.lis"
        raise ValueError
    try:
      filename1 = file[file.rfind('/')+1:]
      self.load_trail_mlist([filename1],mfile)
    except:
      print "Unable to load master list with trail files"
      raise ValueError
    if(file is not None):
      list_steps = []
      try:
        self.load_trail_files([file])
      except:
        print "Unable to load trail files"
        raise ValueError
      filename = file[file.rfind('/')+1:]
      for tag in self.mlist[filename]:
        if self.mlist[filename][tag]['time'] != '':
          list_steps.append(tag)
    else:
      list_keys = {}
      for key in self.mlist:
        list_steps = []
        list_keys[key] = []
        for tag in self.mlist[key]:
          list_steps.append(tag)
        list_steps.sort()
        list_keys[key] = list_steps
    return list_steps

  ##############################################################################

  def cmp_steps(self,file1,file2,mfile=None):
    """ Will compare the steps of each file and return the number of differences

      Parameters:

      mfile : Master file for complete list of steps
      file1 : File to compare
      file2 : File to compare

      Return Values:

      no_diff : Number of differences found

    """
    
    if(mfile == None):
      try:
        mfile = evaluateDefinitionsFile("trailer_master.lis")
      except:
        print "Unable to determine trailer_master.lis"
        raise ValueError
    try:
      filename1 = file1[file1.rfind('/')+1:]
      filename2 = file2[file2.rfind('/')+1:]
      self.load_trail_mlist([filename1,filename2],mfile)
    except:
      print "Unable to load master list with trail files"
      raise ValueError
    try: 
      file1_list = self.list_steps(file1)
    except:
      print "Unable to list steps for: ",file1
      raise ValueError
    try:
      file2_list = self.list_steps(file2)
    except:
      print "Unable to list steps for: ",file2
      raise ValueError

    no_diff = 0

    for step in file1_list:
      found = 0
      for check in file2_list:
        if(check == step):
          found = 1
          break
      if(found == 0):
        print "File " + file1 + " has exclusive step: " + step
        no_diff +=1
    for step in file2_list:
      found = 0
      for check in file1_list:
        if(check == step):
          found = 1
          break
      if(found == 0):
        print "File " + file2 + " has exclusive step: " + step
        no_diff +=1
    return no_diff

################################################################################      
